import archeo.logger
from archeo.preset.aligned_spin import ALIGNED_SPIN_PRIOR
from archeo.preset.precessing import PRECESSING_PRIOR
from archeo.schema import PriorConfig


local_logger = archeo.logger.get_logger(__name__)


PRIOR_STORE = {
    "aligned_spin": ALIGNED_SPIN_PRIOR,
    "precessing": PRECESSING_PRIOR,
}


def get_prior_config(name: str = "default") -> PriorConfig:
    """Get a prior configuration.
    NOTE: By default, it returns the precessing prior.

    Args:
        name (str): The name of the prior.

    Returns:
        prior_config (PriorConfig): The prior configuration.
    """

    if name == "default":
        return PRIOR_STORE["precessing"]

    if name not in PRIOR_STORE:
        msg = f"Prior {name} not found."
        local_logger.error(msg)
        raise ValueError(msg)

    return PRIOR_STORE[name]



from dataclasses import dataclass

import numpy as np
import yaml

from archeo.constants import Fits


@dataclass(frozen=True)
class Domain:
    """Domain of a parameter.

    Attributes:
        low (float): Lower bound of the domain
        high (float): Upper bound of the domain
    """

    low: float = float("-inf")
    high: float = float("inf")

    def contain(self, value: float) -> bool:
        """Check if the input value is within the domain."""

        return self.low <= value <= self.high

    def draw(self) -> float:
        """Draw a random value from the domain."""

        return np.random.uniform(low=self.low, high=self.high)


@dataclass(frozen=True)
class Binary:
    """Binary parameters.

    Attributes:
        m_1 (float): Mass of the primary black hole (in solar mass)
        m_2 (float): Mass of the secondary black hole (in solar mass)
        chi_1 (tuple[float, float, float]): Spin of the primary black hole (dimensionless), [0, 1]
        chi_2 (tuple[float, float, float]): Spin of the secondary black hole (dimensionless), [0, 1]
    """

    m_1: float
    m_2: float
    chi_1: tuple[float, float, float]
    chi_2: tuple[float, float, float]


@dataclass(frozen=True)
class Event:
    """Event (Binary Black Hole Merger) parameters."""

    m_1: float
    m_2: float
    m_ret: float
    v_f: tuple[float, float, float]
    chi_1: tuple[float, float, float]
    chi_2: tuple[float, float, float]
    chi_f: tuple[float, float, float]


@dataclass(frozen=True)
class PriorConfig:
    """Configuration of the prior.

    Attributes:
        n_samples (int): Number of samples to generate.
        fits (Fits): Surrogate model to use.
        is_spin_aligned (bool): Whether the spins are aligned or not.
        is_only_up_aligned_spin (bool): Whether the spins are only in the positive z-direction.
        spin (Domain): Domain of the spin parameter.
        phi (Domain): Domain of the azimuthal angle of the spin.
        theta (Domain): Domain of the polar angle of the spin.
        mass_ratio (Domain): Domain of the mass ratio.
        mass (Domain): Domain of the mass.
        is_mahapatra (bool): Whether the Mahapatra mass function is used.
    """

    n_samples: int
    fits: Fits
    is_spin_aligned: bool
    is_only_up_aligned_spin: bool
    spin: Domain  # unit: dimensionless
    phi: Domain  # unit: pi
    theta: Domain  # unit: pi
    mass_ratio: Domain  # unit: dimensionless
    mass: Domain  # unit: solar mass
    is_mahapatra: bool

    def __post_init__(self) -> None:
        """Post initialization."""

        if self.fits not in Fits:
            raise ValueError(f"Invalid fits: {self.fits}")

        if self.is_only_up_aligned_spin and not self.is_spin_aligned:
            raise ValueError("Only up-aligned spin is only valid when spins are aligned.")

    def from_yaml(self, filepath: str) -> "PriorConfig":
        """Load prior configuration from a yaml file.

        Args:
            filepath (str): Path to the yaml file.

        Returns:
            PriorConfig: Prior configuration.
        """

        with open(filepath, "r", encoding="utf-8") as f:
            config = yaml.safe_load(f)

        return PriorConfig(**config)


@dataclass(frozen=True)
class Padding:
    """
    Padding for plot.

    Attributes:
        tpad: float, the top padding of the plot.
        lpad: float, the left padding of the plot.
        bpad: float, the bottom padding of the plot.
    """

    tpad: float = 2.5
    lpad: float = 0.1
    bpad: float = 0.12


@dataclass(frozen=True)
class Labels:
    """
    Labels for plot.

    Attributes:
        title: str, the title of the plot.
        xlabel: str, the x-axis label of the plot.
        ylabel: str, the y-axis label of the plot.
    """

    title: str = ""
    xlabel: str = ""
    ylabel: str = ""



import enum

import archeo.logger


local_logger = archeo.logger.get_logger(__name__)

SPEED_OF_LIGHT = 299792.458  # km/s


class Fits(enum.Enum):
    """Surrogate models for binary black hole merger simulations.

    Attributes:
        NRSur3dq8Remnant: non precessing BHs with mass ratio<=8, anti-/aligned spin <= 0.8.
        NRSur7dq4Remnant: precessing BHs with mass ratio<=4, generic spin <= 0.8.
        surfinBH7dq2: precessing BHs with mass ratio <= 2, generic spin <= 0.8.

    Details please refer to https://pypi.org/project/surfinBH/.
    """

    NRSUR3DQ8REMNANT = "NRSur3dq8Remnant"
    NRSUR7DQ4REMNANT = "NRSur7dq4Remnant"
    SURFINBH7DQ2 = "surfinBH7dq2"

    def load(self):
        """Load a surfinBH fits.

        Returns:
            fits (surfinBH.surfinBH.SurFinBH): The loaded fits.
        """

        import surfinBH  # pylint: disable=import-outside-toplevel

        local_logger.info(
            "Loading surfinBH %s, description: %s.",
            self.value,
            surfinBH.fits_collection[self.value].desc,
        )
        return surfinBH.LoadFits(self.value)


class EscapeVelocity(enum.Enum):
    """Escape velocity (Unit in km s^-1)"""

    GLOBULAR_CLUSTER = 50.0
    MILKY_WAY = 600.0
    NUCLEAR_STAR_CLUSTER = 1500.0
    ELLIPTICAL_GALAXY = 2500.0

    def label(self):
        """Return the escape velocity label"""

        if self is EscapeVelocity.GLOBULAR_CLUSTER:
            return "$v_{esc, GC}$"

        if self is EscapeVelocity.MILKY_WAY:
            return "$v_{esc, MW}$"

        if self is EscapeVelocity.NUCLEAR_STAR_CLUSTER:
            return "$v_{esc, NSC}$"

        if self is EscapeVelocity.ELLIPTICAL_GALAXY:
            return "$v_{esc, EG}$"

        raise ValueError(f"Unknown escape velocity {self}")

    @classmethod
    def to_vlines(cls) -> dict[str, float]:
        """Return a dictionary for vlines plotting

        Returns:
            vlines (Dict[str, float]): The escape velocity vlines.
        """

        return {esc_vel.label(): esc_vel.value for esc_vel in cls}


class Columns(str, enum.Enum):
    """Columns in the prior dataframe"""

    HEAVIER_BH_MASS = "m_1"
    HEAVIER_BH_SPIN = "a_1"
    HEAVIER_BH_CHI = "chi_1"
    LIGHTER_BH_MASS = "m_2"
    LIGHTER_BH_SPIN = "a_2"
    LIGHTER_BH_CHI = "chi_2"
    MASS_RATIO = "q"
    RETAINED_MASS = "m_ret"
    LIKELIHOOD = "l"
    RECOVERY_RATE = "r_rec"
    BH_MASS = "m_f"
    BH_KICK = "k_f"
    BH_VEL = "v_f"
    BH_CHI = "chi_f"
    BH_SPIN = "a_f"
    BH_EFF_SPIN = "a_eff"
    BH_PREC_SPIN = "a_prec"



from archeo.constants import Fits
from archeo.core.prior import Prior
from archeo.preset import get_prior_config
from archeo.schema import PriorConfig
from archeo.visualization import visualize_posterior_estimation, visualize_prior_distribution



from archeo.constants import Fits
from archeo.schema import Domain, PriorConfig


ALIGNED_SPIN_PRIOR = PriorConfig(
    n_samples=2000000,
    fits=Fits.NRSUR3DQ8REMNANT,
    is_spin_aligned=True,
    is_only_up_aligned_spin=False,
    spin=Domain(low=0.0, high=1.0),
    phi=Domain(low=0.0, high=2.0),
    theta=Domain(low=0.0, high=1.0),
    mass=Domain(low=5.0, high=65.0),
    mass_ratio=Domain(low=1.0, high=6.0),
    is_mahapatra=False,
)



import os
from typing import Any

import yaml

import archeo.logger


local_logger = archeo.logger.get_logger(__name__)


def check_and_create_dir(dirpath: str) -> None:
    """Check if the directory exists, if not create it

    Args:
        dirpath (str): The directory path
    """

    if not os.path.isdir(dirpath):
        os.mkdir(dirpath)

    local_logger.info("Created directory: %s", dirpath)


def load_yml(filepath: str) -> dict[str, Any]:
    """Load a yml file

    Args:
        filepath (str): The path to the yml file

    Returns:
        dict[str, Any]: The content of the yml file
    """

    local_logger.info("Read yml: %s", filepath)
    with open(filepath, mode="r", encoding="utf-8") as file:
        content = yaml.load(file, Loader=yaml.SafeLoader)

    local_logger.debug("Yml content: %s", content)
    return content


def save_as_yml(filepath: str, content: dict[str, Any]) -> None:
    """Save content as yml

    Args:
        filepath (str): The path to save the yml file
        content (dict[str, Any]): The content to save
    """

    with open(filepath, mode="w", encoding="utf-8") as file:
        yaml.dump(content, file)

    local_logger.info("Saved config at %s", filepath)
